from karabo.bound import Types
from karabo.common.scenemodel.api import (
    DisplayCommandModel,
    DisplayLabelModel,
    DisplayStateColorModel,
    DisplayTextLogModel,
    DoubleLineEditModel,
    IntLineEditModel,
    LabelModel,
    SceneModel,
    write_scene,
)

BASE_INC = 30
PADDING = 10
RECONFIGURABLE = 4  # TODO: look up proper enum


def correction_device_constants_scene(device_id, schema, param_prefix, status_prefix):
    # TODO: are there layout models in scene model somewhere? like gridbox?
    subscenes = []
    max_height = 0
    # first column: parameters
    row_offset = PADDING
    col_offset = PADDING
    subscenes.append(
        LabelModel(
            text="Parameters used for CalCat queries",
            width=12 * BASE_INC,
            x=col_offset,
            y=row_offset,
        )
    )
    row_offset += BASE_INC
    for key in schema.getKeys(param_prefix):
        key_path = f"{param_prefix}.{key}"
        subscenes.append(
            LabelModel(
                text=schema.getDisplayedName(key_path)
                if schema.hasDisplayedName(key_path)
                else key,
                width=5 * BASE_INC,
                height=BASE_INC,
                x=col_offset,
                y=row_offset,
            )
        )
        subscenes.append(
            DisplayLabelModel(
                keys=[f"{device_id}.{param_prefix}.{key}"],
                width=7 * BASE_INC,
                height=BASE_INC,
                x=col_offset + 5 * BASE_INC,
                y=row_offset,
            )
        )
        if schema.getAccessMode(key_path) == RECONFIGURABLE:
            value_type = schema.getValueType(key_path)
            if value_type in (Types.DOUBLE, Types.FLOAT):
                subscenes.append(
                    DoubleLineEditModel(
                        keys=[f"{device_id}.{param_prefix}.{key}"],
                        width=7 * BASE_INC,
                        height=BASE_INC,
                        x=col_offset + 12 * BASE_INC,
                        y=row_offset,
                    )
                )
            elif value_type in (Types.INT32, Types.UINT32, Types.INT64, Types.UINT64):
                subscenes.append(
                    IntLineEditModel(
                        keys=[f"{device_id}.{param_prefix}.{key}"],
                        width=7 * BASE_INC,
                        height=BASE_INC,
                        x=col_offset + 12 * BASE_INC,
                        y=row_offset,
                    )
                )
            else:
                subscenes.append(
                    LabelModel(
                        text=f"Not yet supported: editing {key} of type {value_type}",
                        width=7 * BASE_INC,
                        height=BASE_INC,
                        x=col_offset + 12 * BASE_INC,
                        y=row_offset,
                    )
                )

        row_offset += BASE_INC
    max_height = max(max_height, row_offset)

    # second column: constants as currently loaded
    row_offset = PADDING
    col_offset += 19 * BASE_INC  # width of first column
    col_offset += PADDING  # and padding
    subscenes.append(
        DisplayCommandModel(
            keys=[f"{device_id}.loadMostRecentConstants"],
            x=col_offset,
            y=row_offset,
            width=10 * BASE_INC,
            height=BASE_INC,
        )
    )
    row_offset += BASE_INC
    subscenes.append(
        LabelModel(
            text="Constants found and loaded",
            width=10 * BASE_INC,
            height=BASE_INC,
            x=col_offset,
            y=row_offset,
        )
    )
    row_offset += BASE_INC
    for constant_name in schema.getKeys(status_prefix):
        constant_path = f"{status_prefix}.{constant_name}"
        subscenes.append(
            LabelModel(
                text=constant_name,
                width=10 * BASE_INC,
                height=BASE_INC,
                x=col_offset,
                y=row_offset,
            )
        )
        row_offset += BASE_INC
        for field in ("found", "createdAt"):
            subscenes.append(
                LabelModel(
                    text=field,  # TODO: displayedName
                    width=5 * BASE_INC,
                    height=BASE_INC,
                    x=col_offset,
                    y=row_offset,
                )
            )
            subscenes.append(
                DisplayLabelModel(
                    keys=[f"{device_id}.{constant_path}.{field}"],
                    width=7 * BASE_INC,
                    height=BASE_INC,
                    x=col_offset + 5 * BASE_INC,
                    y=row_offset,
                )
            )
            row_offset += BASE_INC
    max_height = max(max_height, row_offset)

    scene = SceneModel(
        height=max_height + 2 * PADDING,
        width=col_offset + 12 * BASE_INC + PADDING,
        children=subscenes,
    )
    return write_scene(scene)


def correction_device_overview_scene(device_id):
    subscenes = []
    row_offset = 0
    # device class and name
    subscenes.append(
        LabelModel(
            text="Correction device",
            width=5 * BASE_INC,
            height=BASE_INC,
            x=0,
            y=row_offset,
        )
    )
    subscenes.append(
        DisplayLabelModel(
            keys=[f"{device_id}.deviceId"],
            width=10 * BASE_INC,
            height=BASE_INC,
            x=5 * BASE_INC,
            y=row_offset,
        )
    )
    row_offset += BASE_INC
    # device state
    subscenes.append(
        DisplayStateColorModel(
            keys=[f"{device_id}.state"],
            width=5 * BASE_INC,
            height=BASE_INC,
            # parent_component="DisplayComponent",
            x=0,
            y=row_offset,
        )
    )
    subscenes.append(
        DisplayLabelModel(
            keys=[f"{device_id}.state"],
            width=5 * BASE_INC,
            height=BASE_INC,
            x=0,
            y=row_offset,
        )
    )
    row_offset += BASE_INC
    # log of status
    subscenes.append(
        DisplayTextLogModel(
            keys=[f"{device_id}.status"],
            width=10 * BASE_INC,
            height=10 * BASE_INC,
            x=0,
            y=row_offset,
        )
    )
    row_offset += BASE_INC * 10
    scene = SceneModel(height=845.0, width=742.0, children=subscenes)
    return write_scene(scene)
